package de.ozgcloud.nachrichten.postfach.osiv2.config;

import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.http.client.reactive.ReactorClientHttpConnector;
import org.springframework.security.oauth2.client.AuthorizedClientServiceReactiveOAuth2AuthorizedClientManager;
import org.springframework.security.oauth2.client.InMemoryReactiveOAuth2AuthorizedClientService;
import org.springframework.security.oauth2.client.ReactiveOAuth2AuthorizedClientProvider;
import org.springframework.security.oauth2.client.ReactiveOAuth2AuthorizedClientProviderBuilder;
import org.springframework.security.oauth2.client.endpoint.WebClientReactiveClientCredentialsTokenResponseClient;
import org.springframework.security.oauth2.client.registration.ReactiveClientRegistrationRepository;
import org.springframework.security.oauth2.client.web.reactive.function.client.ServerOAuth2AuthorizedClientExchangeFilterFunction;
import org.springframework.util.LinkedMultiValueMap;
import org.springframework.util.MultiValueMap;
import org.springframework.web.reactive.function.client.WebClient;

import de.ozgcloud.nachrichten.postfach.osiv2.gen.ApiClient;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.api.MessageExchangeApi;
import lombok.RequiredArgsConstructor;
import reactor.netty.http.client.HttpClient;
import reactor.netty.transport.ProxyProvider;

@Configuration
@RequiredArgsConstructor
@ConditionalOnProperty("ozgcloud.osiv2-postfach.enabled")
public class ApiClientConfiguration {

	private final OsiPostfachProperties.ApiConfiguration apiConfiguration;
	private final OsiPostfachProperties.ProxyConfiguration proxyConfiguration;

	@Bean
	MessageExchangeApi messageExchangeApi(ApiClient apiClient) {
		return new MessageExchangeApi(apiClient);
	}

	@Bean
	ApiClient apiClient(ReactiveClientRegistrationRepository clientRegistrations) {
		return new ApiClient(osi2PostfachWebClient(clientRegistrations))
				.setBasePath(apiConfiguration.getUrl());
	}

	private WebClient osi2PostfachWebClient(
			ReactiveClientRegistrationRepository clientRegistrations) {
		return WebClient.builder()
				.clientConnector(new ReactorClientHttpConnector(httpClient()))
				.filter(serverOAuth2AuthorizedClientExchangeFilterFunction(clientRegistrations))
				.build();
	}

	@SuppressWarnings("ConstantConditions")
	private HttpClient httpClient() {
		var webClient = HttpClient.create();
		return proxyConfiguration.isEnabled() ? webClient
				.proxy(proxy -> proxy
						.type(ProxyProvider.Proxy.HTTP)
						.host(proxyConfiguration.getHost())
						.port(proxyConfiguration.getPort())
						.username(proxyConfiguration.getUsername())
						.password(username -> proxyConfiguration.getPassword())
				) : webClient;
	}

	private ServerOAuth2AuthorizedClientExchangeFilterFunction serverOAuth2AuthorizedClientExchangeFilterFunction(
			ReactiveClientRegistrationRepository clientRegistrations) {

		var oauth = new ServerOAuth2AuthorizedClientExchangeFilterFunction(authorizedClientManager(clientRegistrations));
		oauth.setDefaultClientRegistrationId("osi2");
		return oauth;
	}

	private AuthorizedClientServiceReactiveOAuth2AuthorizedClientManager authorizedClientManager(
			ReactiveClientRegistrationRepository clientRegistrations) {
		var clientService = new InMemoryReactiveOAuth2AuthorizedClientService(
				clientRegistrations);
		var authorizedClientManager = new AuthorizedClientServiceReactiveOAuth2AuthorizedClientManager(
				clientRegistrations, clientService);

		authorizedClientManager.setAuthorizedClientProvider(authorizedClientProvider());

		return authorizedClientManager;
	}

	private ReactiveOAuth2AuthorizedClientProvider authorizedClientProvider() {
		return ReactiveOAuth2AuthorizedClientProviderBuilder.builder()
				.clientCredentials(builder ->
						builder.accessTokenResponseClient(clientCredentialsTokenResponseClient())
				)
				.build();
	}

	private WebClientReactiveClientCredentialsTokenResponseClient clientCredentialsTokenResponseClient() {
		var client = new WebClientReactiveClientCredentialsTokenResponseClient();
		configureHttpClientForTokenRequests(client);
		configureParametersForTokenRequests(client);
		return client;
	}

	private void configureHttpClientForTokenRequests(WebClientReactiveClientCredentialsTokenResponseClient client) {
		client.setWebClient(WebClient.builder()
				.clientConnector(new ReactorClientHttpConnector(httpClient()))
				.build());
	}

	private void configureParametersForTokenRequests(WebClientReactiveClientCredentialsTokenResponseClient client) {
		client.addParametersConverter(source -> {
			MultiValueMap<String, String> parameters = new LinkedMultiValueMap<>();
			// Pass a resource indicator parameter https://datatracker.ietf.org/doc/html/rfc8707
			parameters.add("resource", apiConfiguration.getResource());
			return parameters;
		});
	}

}
