package de.ozgcloud.nachrichten.postfach.osiv2.transfer;

import static de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachAddressTestFactory.*;
import static de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory.*;
import static org.assertj.core.api.Assertions.*;

import java.util.stream.Stream;

import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.Arguments;
import org.junit.jupiter.params.provider.MethodSource;
import org.mapstruct.factory.Mappers;

import de.ozgcloud.nachrichten.postfach.PostfachAddress;
import de.ozgcloud.nachrichten.postfach.PostfachAddressIdentifier;
import de.ozgcloud.nachrichten.postfach.PostfachNachricht;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachAddressTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.factory.PostfachNachrichtTestFactory;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.OutSendMessageRequestV2;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1EidasLevel;
import de.ozgcloud.nachrichten.postfach.osiv2.gen.model.V1ReplyBehavior;

class RequestMapperTest {

	private final RequestMapper mapper = Mappers.getMapper(RequestMapper.class);

	@DisplayName("map mailbox id")
	@Nested
	class TestMapMailboxId {

		@DisplayName("should map postfach address to mailbox id")
		@Test
		void shouldMapPostfachAddressToMailboxId() {
			var result = mapper.mapMailboxId(PostfachNachrichtTestFactory.create());

			assertThat(result).isEqualTo(MAILBOX_ID);
		}

		@DisplayName("should throw on missing postfach address")
		@ParameterizedTest
		@MethodSource("badPostfachAddresses")
		void shouldThrowOnMissingPostfachAddress(PostfachAddress badPostfachAddress) {
			var nachrichtWithBadPostfachAddress = PostfachNachrichtTestFactory.createBuilder()
					.postfachAddress(badPostfachAddress)
					.build();

			assertThatThrownBy(() -> mapper.mapMailboxId(nachrichtWithBadPostfachAddress))
					.isInstanceOf(IllegalArgumentException.class);
		}

		static Stream<Arguments> badPostfachAddresses() {
			return Stream.of(
					null,
					Arguments.of(PostfachAddressTestFactory.createBuilder()
							.identifier(null)
							.build()),
					Arguments.of(PostfachAddressTestFactory.createBuilder()
							.identifier(new PostfachAddressIdentifier() {

							})
							.build())
			);
		}
	}

	@DisplayName("map OutSendMessageRequestV2")
	@Nested
	class TestMapOutSendMessageRequestV2 {

		@DisplayName("should map sequence number")
		@Test
		void shouldMapSequenceNumber() {
			var result = doMapping();

			assertThat(result.getSequencenumber()).isEqualTo(VORGANG_ID);
		}

		@DisplayName("should map subject")
		@Test
		void shouldMapSubject() {
			var result = doMapping();

			assertThat(result.getSubject()).isEqualTo(MAIL_SUBJECT);
		}

		@DisplayName("should map body")
		@Test
		void shouldMapBody() {
			var result = doMapping();

			assertThat(result.getBody()).isEqualTo(MAIL_BODY);
		}

		@DisplayName("should map display name")
		@Test
		void shouldMapDisplayName() {
			var result = doMapping();

			assertThat(result.getDisplayName()).isEqualTo(USER_ID);
		}

		@DisplayName("should map origin sender to null")
		@Test
		void shouldMapOriginSenderToNull() {
			var result = doMapping();

			assertThat(result.getOriginSender()).isNull();
		}

		@DisplayName("should map forbidden reply action")
		@Test
		void shouldMapForbiddenReplyAction() {
			var result = doMapping();

			assertThat(result.getReplyAction()).isEqualTo(V1ReplyBehavior.REPLYFORBIDDEN);
		}

		@DisplayName("should map possible reply action")
		@Test
		void shouldMapPossibleReplyAction() {
			var nachricht = PostfachNachrichtTestFactory.createBuilder()
					.replyOption(PostfachNachricht.ReplyOption.POSSIBLE)
					.build();

			var result = mapper.mapOutSendMessageRequestV2(nachricht);

			assertThat(result.getReplyAction()).isEqualTo(V1ReplyBehavior.REPLYPOSSIBLE);
		}

		@DisplayName("should map mandatory reply action")
		@Test
		void shouldMapMandatoryReplyAction() {
			var nachricht = PostfachNachrichtTestFactory.createBuilder()
					.replyOption(PostfachNachricht.ReplyOption.MANDATORY)
					.build();

			var result = mapper.mapOutSendMessageRequestV2(nachricht);

			assertThat(result.getReplyAction()).isEqualTo(V1ReplyBehavior.REPLYMANDATORY);
		}

		@DisplayName("should set eidasLevel")
		@Test
		void shouldSetEidasLevel() {
			var result = doMapping();

			assertThat(result.getEidasLevel()).isEqualTo(V1EidasLevel.LOW);
		}

		@DisplayName("should set isObligatory")
		@Test
		void shouldSetIsObligatory() {
			var result = doMapping();

			assertThat(result.getIsObligatory()).isFalse();
		}

		@DisplayName("should set isHtml")
		@Test
		void shouldSetIsHtml() {
			var result = doMapping();

			assertThat(result.getIsHtml()).isFalse();
		}

		@DisplayName("should map files")
		@Test
		void shouldMapFiles() {
			var result = doMapping();

			assertThat(result.getFiles()).isEmpty();
		}

		@DisplayName("should map references")
		@Test
		void shouldMapReferences() {
			var result = doMapping();

			assertThat(result.getReferences()).isEmpty();
		}

		private OutSendMessageRequestV2 doMapping() {
			return mapper.mapOutSendMessageRequestV2(PostfachNachrichtTestFactory.create());
		}
	}

}